@@ -31,18 +31,20 @@ import logging
 import os
 import re
 import warnings
-from subprocess import Popen, PIPE
 from time import time, strptime, mktime, sleep
+from typing import Tuple, Union, List, Dict
 
 # pySMART module imports
 from .attribute import Attribute
+from .diagnostics import Diagnostics
 from .testentry import TestEntry
-from .utils import smartctl_type, smartctl_isvalid_type, SMARTCTL_PATH, any_in, all_in
+from .smartctl import Smartctl
+from .utils import smartctl_type, smartctl_isvalid_type, any_in, all_in
 
 logger = logging.getLogger('pySMART')
 
 
-def smart_health_assement(disk_name):
+def smart_health_assement(disk_name, smartctl: Smartctl = Smartctl()):
     """
     This function gets the SMART Health Status of the disk (IF the disk
     is SMART capable and smart is enabled on it else returns None).
@@ -50,15 +52,9 @@ def smart_health_assement(disk_name):
     since in non-abridged mode update gets this information anyways.
     """
     assessment = None
-    cmd = Popen(
-        [SMARTCTL_PATH, '--health',
-            os.path.join('/dev/', disk_name.replace('nvd', 'nvme'))],
-        stdout=PIPE,
-        stderr=PIPE,
-    )
-    _stdout, _stderr = [i.decode('utf8') for i in cmd.communicate()]
-    _stdout = _stdout.split('\n')
-    line = _stdout[4]  # We only need this line
+    raw = smartctl.health(os.path.join(
+        '/dev/', disk_name.replace('nvd', 'nvme')))
+    line = raw[4]  # We only need this line
     if 'SMART overall-health self-assessment' in line:  # ATA devices
         if line.split(':')[1].strip() == 'PASSED':
             assessment = 'PASS'
@@ -79,10 +75,8 @@ class Device(object):
     (considered SATA) but excludes other external devices (USB, Firewire).
     """
 
-    def __init__(self, name, interface=None, abridged=False, smart_options=''):
+    def __init__(self, name: str, interface: str = None, abridged: bool = False, smart_options: Union[str, List[str]] = None, smartctl: Smartctl = Smartctl()):
         """Instantiates and initializes the `pySMART.device.Device`."""
-        if not SMARTCTL_PATH:
-            raise FileNotFoundError("Command smartctl doesn't exist!")
         if not (
                 interface is None or
                 smartctl_isvalid_type(interface.lower())
@@ -90,18 +84,23 @@ class Device(object):
             raise ValueError(
                 'Unknown interface: {0} specified for {1}'.format(interface, name))
         self.abridged = abridged or interface == 'UNKNOWN INTERFACE'
-        self.smart_options = smart_options.split(
-            ' ') if smart_options else ['']
-        self.name = name.replace('/dev/', '').replace('nvd', 'nvme')
+        if smart_options is not None:
+            if isinstance(smart_options,  str):
+                smart_options = smart_options.split(' ')
+            smartctl.add_options(smart_options)
+        self.smartctl = smartctl
+        """
+        """
+        self.name: str = name.replace('/dev/', '').replace('nvd', 'nvme')
         """
         **(str):** Device's hardware ID, without the '/dev/' prefix.
         (ie: sda (Linux), pd0 (Windows))
         """
-        self.model = None
+        self.model: str = None
         """**(str):** Device's model number."""
-        self.serial = None
+        self.serial: str = None
         """**(str):** Device's serial number."""
-        self.interface = None if interface == 'UNKNOWN INTERFACE' else interface
+        self.interface: str = None if interface == 'UNKNOWN INTERFACE' else interface
         """
         **(str):** Device's interface type. Must be one of:
             * **ATA** - Advanced Technology Attachment
@@ -116,40 +115,40 @@ class Device(object):
         occur. Otherwise, this value overrides the auto-detected type and could
         produce unexpected or no data.
         """
-        self.capacity = None
-        """**(str):** Device's user capacity."""
-        self.firmware = None
+        self._capacity: str = None
+        """**(str):** Device's user capacity as reported directly by smartctl (RAW)."""
+        self.firmware: str = None
         """**(str):** Device's firmware version."""
-        self.smart_capable = 'nvme' in self.name
+        self.smart_capable: bool = 'nvme' in self.name
         """
         **(bool):** True if the device has SMART Support Available.
         False otherwise. This is useful for VMs amongst other things.
         """
-        self.smart_enabled = 'nvme' in self.name
+        self.smart_enabled: bool = 'nvme' in self.name
         """
         **(bool):** True if the device supports SMART (or SCSI equivalent) and
         has the feature set enabled. False otherwise.
         """
-        self.assessment = None
+        self.assessment: str = None
         """
         **(str):** SMART health self-assessment as reported by the device.
         """
-        self.messages = []
+        self.messages: List[str] = []
         """
         **(list of str):** Contains any SMART warnings or other error messages
         reported by the device (ie: ascq codes).
         """
-        self.is_ssd = True if 'nvme' in self.name else False
+        self.is_ssd: bool = True if 'nvme' in self.name else False
         """
         **(bool):** True if this device is a Solid State Drive.
         False otherwise.
         """
-        self.rotation_rate = None
+        self.rotation_rate: int = None
         """
         **(int):** The Roatation Rate of the Drive if it is not a SSD.
         The Metric is RPM.
         """
-        self.attributes = [None] * 256
+        self.attributes: List[Attribute] = [None] * 256
         """
         **(list of `Attribute`):** Contains the complete SMART table
         information for this device, as provided by smartctl. Indexed by
@@ -174,11 +173,10 @@ class Device(object):
         # Note: The above are just default values and can/will be changed
         # upon update() when the attributes and type of the disk is actually
         # determined.
-        self.tests = []
+        self.tests: List[TestEntry] = []
         """
-        **(list of `Log_Entry`):** Contains the complete SMART self-test log
-        for this device, as provided by smartctl. If no SMART self-tests have
-        been recorded, contains a `None` type instead.
+        **(list of `TestEntry`):** Contains the complete SMART self-test log
+        for this device, as provided by smartctl.
         """
         self._test_running = False
         """
@@ -194,17 +192,19 @@ class Device(object):
         """
         **(int):** Estimate progress percantage of the running SMART selftest.
         """
-        self.diags = {}
+        self.diagnostics: Diagnostics = Diagnostics()
         """
-        **(dict of str):** Contains parsed and processed diagnostic information
+        **Diagnostics** Contains parsed and processed diagnostic information
         extracted from the SMART information. Currently only populated for
         SAS and SCSI devices, since ATA/SATA SMART attributes are manufacturer
         proprietary.
         """
-        self.temperature = None
+        self.temperature: int = None
         """
         **(int or None): Since SCSI disks do not report attributes like ATA ones
-        we need to grep/regex the shit outta the normal "smartctl -a" output
+        we need to grep/regex the shit outta the normal "smartctl -a" output.
+        In case the device have more than one temperature sensor the first value
+        will be stored here too.
         """
         self.temperatures = {}
         """
@@ -224,14 +224,10 @@ class Device(object):
         elif self.interface is None and not self.abridged:
             logger.trace(
                 "Determining interface of disk: {0}".format(self.name))
-            cmd = Popen(
-                [SMARTCTL_PATH, '-d', 'test',
-                    os.path.join('/dev/', self.name)],
-                stdout=PIPE,
-                stderr=PIPE
-            )
-            _stdout, _stderr = [i.decode('utf8') for i in cmd.communicate()]
-            if _stdout != '':
+            raw, returncode = self.smartctl.generic_call(
+                ['-d', 'test', os.path.join('/dev/', self.name)])
+
+            if len(raw) > 0:
                 # I do not like this parsing logic but it works for now!
                 # just for reference _stdout.split('\n') gets us
                 # something like
@@ -244,7 +240,7 @@ class Device(object):
                 # ]
                 # The above example should be enough for anyone to understand the line below
                 try:
-                    self.interface = _stdout.split('\n')[-2].split("'")[1]
+                    self.interface = raw[-2].split("'")[1]
                     if self.interface == "nvme":  # if nvme set SMART to true
                         self.smart_capable = True
                         self.smart_enabled = True
@@ -267,6 +263,42 @@ class Device(object):
         if self.interface is not None or self.abridged:
             self.update()
 
+    @property
+    def capacity(self) -> str:
+        """Returns the capacity in the raw smartctl format.
+        This may be deprecated in the future and its only retained for compatibility.
+
+        Returns:
+            str: The capacity in the raw smartctl format
+        """
+        return self._capacity
+
+    @property
+    def diags(self) -> Dict[str, str]:
+        """Gets the old/deprecated version of SCSI/SAS diags atribute.
+        """
+        return self.diagnostics.get_classic_format()
+
+    @property
+    def size_raw(self) -> str:
+        """Returns the capacity in the raw smartctl format.
+
+        Returns:
+            str: The capacity in the raw smartctl format
+        """
+        return self._capacity
+
+    @property
+    def size(self) -> int:
+        """Returns the capacity in bytes
+
+        Returns:
+            int: The capacity in bytes
+        """
+        import humanfriendly
+
+        return humanfriendly.parse_size(self._capacity)
+
     def __repr__(self):
         """Define a basic representation of the class object."""
         return "<{0} device on /dev/{1} mod:{2} sn:{3}>".format(
@@ -290,8 +322,8 @@ class Device(object):
             'smart_status': self.assessment,
             'messages': self.messages,
             'test_capabilities': self.test_capabilities.copy(),
-            'tests': [t.__getstate__() for t in self.tests] if self.tests else None,
-            'diagnostics': self.diags.copy(),
+            'tests': [t.__getstate__() for t in self.tests] if self.tests else [],
+            'diagnostics': self.diagnostics.__getstate__(),
             'temperature': self.temperature,
             'attributes': [attr.__getstate__() if attr else None for attr in self.attributes]
         }
@@ -302,7 +334,7 @@ class Device(object):
                 'serial': self.serial,
                 'is_ssd': self.is_ssd,
                 'rotation_rate': self.rotation_rate,
-                'capacity': self.capacity
+                'capacity': self._capacity
             })
         return state_dict
 
@@ -311,16 +343,16 @@ class Device(object):
         del state['smart_status']
         self.__dict__.update(state)
 
-    def smart_toggle(self, action):
+    def smart_toggle(self, action: str) -> Tuple[bool, List[str]]:
         """
         A basic function to enable/disable SMART on device.
 
-        ##Args:
+        # Args:
         * **action (str):** Can be either 'on'(for enabling) or 'off'(for disabling).
 
-        ##Returns"
+        # Returns"
         * **(bool):** Return True (if action succeded) else False
-        * **(str):** None if option succeded else contains the error message.
+        * **(List[str]):** None if option succeded else contains the error message.
         """
         # Lets make the action verb all lower case
         if self.interface == 'nvme':
@@ -336,17 +368,11 @@ class Device(object):
         else:
             if action_lower == 'off':
                 return True, None
-        cmd = Popen(
-            [
-                SMARTCTL_PATH,
-                '-s',
-                action_lower,
-                os.path.join('/dev/', self.name)
-            ],
-            stdout=PIPE, stderr=PIPE)
-        _stdout, _stderr = [i.decode('utf8') for i in cmd.communicate()]
-        if cmd.returncode != 0:
-            return False, _stdout + _stderr
+        raw, returncode = self.smartctl.generic_call(
+            ['-s', action_lower, os.path.join('/dev/', self.name)])
+
+        if returncode != 0:
+            return False, raw
         # if everything worked out so far lets perform an update() and check the result
         self.update()
         if action_lower == 'off' and self.smart_enabled:
@@ -416,56 +442,33 @@ class Device(object):
         if self.interface in ['scsi', 'ata']:
             test = 'sat' if self.interface == 'scsi' else 'sata'
             # Look for a SATA PHY to detect SAT and SATA
-            cmd = Popen(
-                [
-                    SMARTCTL_PATH,
-                    '-d',
-                    smartctl_type(test),
-                    '-l',
-                    'sataphy',
-                    os.path.join('/dev/', self.name)
-                ],
-                stdout=PIPE,
-                stderr=PIPE
-            )
-            _stdout, _stderr = [i.decode('utf8') for i in cmd.communicate()]
-            if 'GP Log 0x11' in _stdout.split('\n')[3]:
+            raw, returncode = self.smartctl.generic_call([
+                '-d',
+                smartctl_type(test),
+                '-l',
+                'sataphy',
+                os.path.join('/dev/', self.name)])
+
+            if 'GP Log 0x11' in raw[3]:
                 self.interface = test
         # If device type is still SCSI (not changed to SAT above), then
         # check for a SAS PHY
         if self.interface == 'scsi':
-            cmd = Popen(
-                [
-                    SMARTCTL_PATH,
-                    '-d',
-                    'scsi',
-                    '-l',
-                    'sasphy',
-                    os.path.join('/dev/', self.name)
-                ],
-                stdout=PIPE,
-                stderr=PIPE
-            )
-            _stdout, _stderr = [i.decode('utf8') for i in cmd.communicate()]
-            if 'SAS SSP' in _stdout.split('\n')[4]:
+            raw, returncode = self.smartctl.generic_call([
+                '-d',
+                'scsi',
+                '-l',
+                'sasphy',
+                os.path.join('/dev/', self.name)])
+            if 'SAS SSP' in raw[4]:
                 self.interface = 'sas'
             # Some older SAS devices do not support the SAS PHY log command.
             # For these, see if smartmontools reports a transport protocol.
             else:
-                cmd = Popen(
-                    [
-                        SMARTCTL_PATH,
-                        '-d',
-                        'scsi',
-                        '-a',
-                        os.path.join('/dev/', self.name)
-                    ],
-                    stdout=PIPE,
-                    stderr=PIPE
-                )
-                _stdout, _stderr = [i.decode('utf8', 'ignore')
-                                    for i in cmd.communicate()]
-                for line in _stdout.split('\n'):
+                raw = self.smartctl.all(
+                    'scsi', os.path.join('/dev/', self.name))
+
+                for line in raw:
                     if 'Transport protocol' in line and 'SAS' in line:
                         self.interface = 'sas'
 
@@ -510,12 +513,12 @@ class Device(object):
         the latest test results. If a new test result is obtained, its content
         is returned.
 
-        ###Args:
+        # Args:
         * **output (str, optional):** If set to 'str', the string
         representation of the most recent test result will be returned, instead
         of a `Test_Entry` object.
 
-        ##Returns:
+        # Returns:
         * **(int):** Return status code. One of the following:
             * 0 - Success. Object (or optionally, string rep) is attached.
             * 1 - Self-test in progress. Must wait for it to finish.
@@ -555,7 +558,7 @@ class Device(object):
         # If so return the newest test result
         # If not, because it's max size already, check for new entries
         if (
-                (self.tests is not None and len(self.tests) != _len) or
+                (len(self.tests) != _len) or
                 (
                     len == maxlog and
                     (
@@ -581,24 +584,12 @@ class Device(object):
         will  abort the Offline Immediate Test routine only if your disk
         has the "Abort Offline collection upon new command"  capability.
 
-        ##Args: Nothing (just aborts directly)
+        # Args: Nothing (just aborts directly)
 
-        ##Returns:
+        # Returns:
         * **(int):** The returncode of calling `smartctl -X device_path`
         """
-        cmd = Popen(
-            [
-                SMARTCTL_PATH,
-                '-d',
-                smartctl_type(self.interface),
-                '-X',
-                os.path.join('/dev/', self.name),
-            ],
-            stdout=PIPE,
-            stderr=PIPE
-        )
-        cmd.wait()
-        return cmd.returncode
+        return self.smartctl.test_stop(smartctl_type(self.interface), os.path.join('/dev/', self.name))
 
     def run_selftest(self, test_type, ETA_type='date'):
         """
@@ -606,7 +597,7 @@ class Device(object):
         'offline' / 'background' mode, allowing normal use of the device while
         it is being tested.
 
-        ##Args:
+        # Args:
         * **test_type (str):** The type of test to run. Accepts the following
         (not case sensitive):
             * **short** - Brief electo-mechanical functionality check.
@@ -627,7 +618,7 @@ class Device(object):
             in. Default is 'date'. One could otherwise specidy 'seconds'.
             Again only for ATA devices.
 
-        ##Returns:
+        # Returns:
         * **(int):** Return status code.  One of the following:
             * 0 - Self-test initiated successfully
             * 1 - Previous self-test running. Must wait for it to finish.
@@ -661,22 +652,12 @@ class Device(object):
                 )
         except KeyError:
             return 2, "Unknown test type '{0}' requested.".format(test_type), None
-        cmd = Popen(
-            [
-                SMARTCTL_PATH,
-                '-d',
-                interface,
-                '-t',
-                test_type,
-                os.path.join('/dev/', self.name)
-            ],
-            stdout=PIPE,
-            stderr=PIPE
-        )
-        _stdout, _stderr = [i.decode('utf8') for i in cmd.communicate()]
+
+        raw = self.smartctl.test_start(
+            interface, test_type, os.path.join('/dev/', self.name))
         _success = False
         _running = False
-        for line in _stdout.split('\n'):
+        for line in raw:
             if 'has begun' in line:
                 _success = True
                 self._test_running = True
@@ -713,7 +694,7 @@ class Device(object):
         (if any errors found) and updates the SMART attributes. Other
         than that it is not visibile anywhere else, so we start it and
         simply return.
-        ##Args:
+        # Args:
         * **test_type (str):** The type of test to run. Accepts the following
         (not case sensitive):
             * **short** - Brief electo-mechanical functionality check.
@@ -738,7 +719,7 @@ class Device(object):
         * **progress_handler (function, optional):** This if provided is called
             with self._test_progress as the supplied argument everytime a poll to
             check the status of the selftest is done.
-        ##Returns:
+        # Returns:
         * **(int):** Return status code.  One of the following:
             * 0 - Self-test executed and finished successfully
             * 1 - Previous self-test running. Must wait for it to finish.
@@ -796,27 +777,13 @@ class Device(object):
         self.temperatures = {}
         if self.abridged:
             interface = None
-            popen_list = [
-                SMARTCTL_PATH,
-                *self.smart_options,
-                '-i',
-                os.path.join('/dev/', self.name)
-            ]
+            raw = self.smartctl.info(os.path.join('/dev/', self.name))
+
         else:
             interface = smartctl_type(self.interface)
-            popen_list = [
-                SMARTCTL_PATH,
-                '-d',
-                interface,
-                *self.smart_options,
-                '-a',
-                os.path.join('/dev/', self.name)
-            ]
-        popen_list = list(filter(None, popen_list))
-        logger.trace("Executing the following cmd: {0}".format(popen_list))
-        cmd = Popen(popen_list, stdout=PIPE, stderr=PIPE)
-        _stdout, _stderr = [i.decode('utf8', 'ignore')
-                            for i in cmd.communicate()]
+            raw = self.smartctl.all(
+                interface, os.path.join('/dev/', self.name))
+
         parse_self_tests = False
         parse_running_test = False
         parse_ascq = False
@@ -825,13 +792,11 @@ class Device(object):
         self._test_running = False
         self._test_progress = None
         # Lets skip the first couple of non-useful lines
-        _stdout = _stdout.split('\n')[4:]
+        _stdout = raw[4:]
         for line in _stdout:
             if line.strip() == '':  # Blank line stops sub-captures
                 if parse_self_tests is True:
                     parse_self_tests = False
-                    if len(self.tests) == 0:
-                        self.tests = None
                 if parse_ascq:
                     parse_ascq = False
                     self.messages.append(message)
@@ -851,12 +816,12 @@ class Device(object):
                 #      Description                              number   (hours)
                 # # 1  Background short  Completed                   -   33124                 - [-   -    -]
                 format_scsi = re.compile(
-                    '^[#\s]*([^\s]+)\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s+\[([^\s]+)\s+([^\s]+)\s+([^\s]+)\]$').match(line)
+                    r'^[#\s]*([^\s]+)\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s+\[([^\s]+)\s+([^\s]+)\s+([^\s]+)\]$').match(line)
 
                 if format_scsi is not None:
                     format = 'scsi'
                     parsed = format_scsi.groups()
-                    num = parsed[0]
+                    num = int(parsed[0])
                     test_type = parsed[1]
                     status = parsed[2]
                     segment = parsed[3]
@@ -885,7 +850,7 @@ class Device(object):
                     # # 1  Extended offline    Completed without error       00%     46660         -
                     format = 'ata'
                     parsed = re.compile(
-                        '^[#\s]*([^\s]+)\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])$').match(line).groups()
+                        r'^[#\s]*([^\s]+)\s{2,}(.*[^\s])\s{2,}(.*[^\s])\s{1,}(.*[^\s])\s{2,}(.*[^\s])\s{2,}(.*[^\s])$').match(line).groups()
                     num = parsed[0]
                     test_type = parsed[1]
                     status = parsed[2]
@@ -914,7 +879,7 @@ class Device(object):
                 self.firmware = line.split(':')[1].strip()
             if any_in(line, 'User Capacity', 'Namespace 1 Size/Capacity'):
                 # TODO: support for multiple NVMe namespaces
-                self.capacity = line.replace(']', '[').split('[')[1].strip()
+                self._capacity = line.replace(']', '[').split('[')[1].strip()
             if 'SMART support' in line:
                 # self.smart_capable = 'Available' in line
                 # self.smart_enabled = 'Enabled' in line
@@ -980,8 +945,7 @@ class Device(object):
                 line_ = ' '.join(line.split()).split(' ')
                 if '' not in line_:
                     self.attributes[int(line_[0])] = Attribute(
-                        line_[0], line_[1], line[2], line_[3], line_[4],
-                        line_[5], line_[6], line_[7], line_[8], line_[9])
+                        int(line_[0]), line_[1], int(line[2], base=16), line_[3], line_[4], line_[5], line_[6], line_[7], line_[8], line_[9])
             # For some reason smartctl does not show a currently running test
             # for 'ATA' in the Test log so I just have to catch it this way i guess!
             # For 'scsi' I still do it since it is the only place I get % remaining in scsi
@@ -1010,75 +974,75 @@ class Device(object):
                 parse_running_test = False
             if all_in(line, 'Description', '(hours)'):
                 parse_self_tests = True  # Set flag to capture test entries
-            if 'No self-tests have been logged' in line:
-                self.tests = None
             # Everything from here on is parsing SCSI information that takes
             # the place of similar ATA SMART information
             if 'used endurance' in line:
                 pct = int(line.split(':')[1].strip()[:-1])
-                self.diags['Life_Left'] = str(100 - pct) + '%'
+                self.diagnostics.Life_Left = 100 - pct
             if 'Specified cycle count' in line:
-                self.diags['Start_Stop_Spec'] = line.split(':')[1].strip()
-                if self.diags['Start_Stop_Spec'] == '0':
-                    self.diags['Start_Stop_Pct_Left'] = '-'
+                self.diagnostics.Start_Stop_Spec = int(
+                    line.split(':')[1].strip())
             if 'Accumulated start-stop cycles' in line:
-                self.diags['Start_Stop_Cycles'] = line.split(':')[1].strip()
-                if 'Start_Stop_Pct_Left' not in self.diags:
-                    self.diags['Start_Stop_Pct_Left'] = str(int(round(
-                        100 - (int(self.diags['Start_Stop_Cycles']) /
-                               int(self.diags['Start_Stop_Spec'])), 0))) + '%'
+                self.diagnostics.Start_Stop_Cycles = int(
+                    line.split(':')[1].strip())
+                if self.diagnostics.Start_Stop_Spec != 0:
+                    self.diagnostics.Start_Stop_Pct_Left = int(round(
+                        100 - (self.diagnostics.Start_Stop_Cycles /
+                               self.diagnostics.Start_Stop_Spec), 0))
             if 'Specified load-unload count' in line:
-                self.diags['Load_Cycle_Spec'] = line.split(':')[1].strip()
-                if self.diags['Load_Cycle_Spec'] == '0':
-                    self.diags['Load_Cycle_Pct_Left'] = '-'
+                self.diagnostics.Load_Cycle_Spec = int(
+                    line.split(':')[1].strip())
             if 'Accumulated load-unload cycles' in line:
-                self.diags['Load_Cycle_Count'] = line.split(':')[1].strip()
-                if 'Load_Cycle_Pct_Left' not in self.diags:
-                    self.diags['Load_Cycle_Pct_Left'] = str(int(round(
-                        100 - (int(self.diags['Load_Cycle_Count']) /
-                               int(self.diags['Load_Cycle_Spec'])), 0))) + '%'
+                self.diagnostics.Load_Cycle_Count = int(
+                    line.split(':')[1].strip())
+                if self.diagnostics.Load_Cycle_Spec != 0:
+                    self.diagnostics.Load_Cycle_Pct_Left = int(round(
+                        100 - (self.diagnostics.Load_Cycle_Count /
+                               self.diagnostics.Load_Cycle_Spec), 0))
             if 'Elements in grown defect list' in line:
-                self.diags['Reallocated_Sector_Ct'] = line.split(':')[
-                    1].strip()
-            if 'read:' in line and interface == 'scsi':
+                self.diagnostics.Reallocated_Sector_Ct = int(
+                    line.split(':')[1].strip())
+            if 'read:' in line:
                 line_ = ' '.join(line.split()).split(' ')
                 if line_[1] == '0' and line_[2] == '0' and line_[3] == '0' and line_[4] == '0':
-                    self.diags['Corrected_Reads'] = '0'
+                    self.diagnostics.Corrected_Reads = 0
                 elif line_[4] == '0':
-                    self.diags['Corrected_Reads'] = str(
-                        int(line_[1]) + int(line_[2]) + int(line_[3]))
+                    self.diagnostics.Corrected_Reads = int(
+                        line_[1]) + int(line_[2]) + int(line_[3])
                 else:
-                    self.diags['Corrected_Reads'] = line_[4]
-                self.diags['Reads_GB'] = line_[6]
-                self.diags['Uncorrected_Reads'] = line_[7]
-            if 'write:' in line and interface == 'scsi':
+                    self.diagnostics.Corrected_Reads = int(line_[4])
+                self.diagnostics.Reads_GB = float(line_[6])
+                self.diagnostics.Uncorrected_Reads = int(line_[7])
+            if 'write:' in line:
                 line_ = ' '.join(line.split()).split(' ')
                 if (line_[1] == '0' and line_[2] == '0' and
                         line_[3] == '0' and line_[4] == '0'):
-                    self.diags['Corrected_Writes'] = '0'
+                    self.diagnostics.Corrected_Writes = 0
                 elif line_[4] == '0':
-                    self.diags['Corrected_Writes'] = str(
-                        int(line_[1]) + int(line_[2]) + int(line_[3]))
+                    self.diagnostics.Corrected_Writes = int(
+                        line_[1]) + int(line_[2]) + int(line_[3])
                 else:
-                    self.diags['Corrected_Writes'] = line_[4]
-                self.diags['Writes_GB'] = line_[6]
-                self.diags['Uncorrected_Writes'] = line_[7]
-            if 'verify:' in line and interface == 'scsi':
+                    self.diagnostics.Corrected_Writes = int(line_[4])
+                self.diagnostics.Writes_GB = float(line_[6])
+                self.diagnostics.Uncorrected_Writes = int(line_[7])
+            if 'verify:' in line:
                 line_ = ' '.join(line.split()).split(' ')
                 if (line_[1] == '0' and line_[2] == '0' and
                         line_[3] == '0' and line_[4] == '0'):
-                    self.diags['Corrected_Verifies'] = '0'
+                    self.diagnostics.Corrected_Verifies = 0
                 elif line_[4] == '0':
-                    self.diags['Corrected_Verifies'] = str(
-                        int(line_[1]) + int(line_[2]) + int(line_[3]))
+                    self.diagnostics.Corrected_Verifies = int(
+                        line_[1]) + int(line_[2]) + int(line_[3])
                 else:
-                    self.diags['Corrected_Verifies'] = line_[4]
-                self.diags['Verifies_GB'] = line_[6]
-                self.diags['Uncorrected_Verifies'] = line_[7]
+                    self.diagnostics.Corrected_Verifies = int(line_[4])
+                self.diagnostics.Verifies_GB = float(line_[6])
+                self.diagnostics.Uncorrected_Verifies = int(line_[7])
             if 'non-medium error count' in line:
-                self.diags['Non-Medium_Errors'] = line.split(':')[1].strip()
+                self.diagnostics.Non_Medium_Errors = int(
+                    line.split(':')[1].strip())
             if 'Accumulated power on time' in line:
-                self.diags['Power_On_Hours'] = line.split(':')[1].split(' ')[1]
+                self.diagnostics.Power_On_Hours = int(
+                    line.split(':')[1].split(' ')[1])
             if 'Current Drive Temperature' in line or ('Temperature:' in
                                                        line and interface == 'nvme'):
                 try:
@@ -1095,6 +1059,8 @@ class Device(object):
                         tempsensor_number = int(tempsensor_number_s)
                         tempsensor_value = int(tempsensor_value_s)
                         self.temperatures[tempsensor_number] = tempsensor_value
+                        if self.temperature is None or tempsensor_number == 0:
+                            self.temperature = tempsensor_value
                 except ValueError:
                     pass
         if not self.abridged:
@@ -1103,40 +1069,22 @@ class Device(object):
                 # corresponding warnings for non-SCSI disks
                 self._make_smart_warnings()
             else:
-                # For SCSI disks, any diagnostic attribute which was not captured
-                # above gets set to '-' to indicate unsupported/unavailable.
-                for diag in ['Corrected_Reads', 'Corrected_Writes',
-                             'Corrected_Verifies', 'Uncorrected_Reads',
-                             'Uncorrected_Writes', 'Uncorrected_Verifies',
-                             'Reallocated_Sector_Ct',
-                             'Start_Stop_Spec', 'Start_Stop_Cycles',
-                             'Load_Cycle_Spec', 'Load_Cycle_Count',
-                             'Start_Stop_Pct_Left', 'Load_Cycle_Pct_Left',
-                             'Power_On_Hours', 'Life_Left', 'Non-Medium_Errors',
-                             'Reads_GB', 'Writes_GB', 'Verifies_GB']:
-                    if diag not in self.diags:
-                        self.diags[diag] = '-'
-                # If not obtained above, make a direct attempt to extract power on
+                # If not obtained Power_On_Hours above, make a direct attempt to extract power on
                 # hours from the background scan results log.
-                if self.diags['Power_On_Hours'] == '-':
-                    cmd = Popen(
+                if self.diagnostics.Power_On_Hours is None:
+                    raw, returncode = self.smartctl.generic_call(
                         [
-                            SMARTCTL_PATH,
                             '-d',
                             'scsi',
                             '-l',
                             'background',
                             os.path.join('/dev/', self.name)
-                        ],
-                        stdout=PIPE,
-                        stderr=PIPE
-                    )
-                    _stdout, _stderr = [i.decode('utf8')
-                                        for i in cmd.communicate()]
-                    for line in _stdout.split('\n'):
+                        ])
+
+                    for line in raw:
                         if 'power on time' in line:
-                            self.diags['Power_On_Hours'] = line.split(':')[
-                                1].split(' ')[1]
+                            self.diagnostics.Power_On_Hours = int(
+                                line.split(':')[1].split(' ')[1])
         # map temperature
         if self.temperature is None:
             # in this case the disk is probably ata
